#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion
/*  ------------------------------------------------------------------------
Colorized Stochastic indicator with one plot and time configurable alerts

    G. Eric Morgan (aka radi8 in Big Mike Trading forum)
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/gpl.txt


	*/
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    ///
    /// </summary>
    [Description("Colorized Stochastic indicator with one plot and time configurable alerts")]
	[Gui.Design.DisplayName("Colorized Stochastic indicator with one plot and time configurable alerts")]
    public class gemStochasticscolor : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		
		private int				periodD	= 3;	// SlowDperiod
		private int				periodK	= 14;	// Kperiod
		private int				smooth	= 3;	// SlowKperiod
		private DataSeries		den;
		private DataSeries		nom;
		private	DataSeries		d;
		private	DataSeries		k;
		private double thresholdOB = 80;
		private double thresholdOS = 20;
		private Color upColor = Color.Green;
		private Color downColor = Color.Red;
		private Color thresholdColorOB  = Color.IndianRed;
		private Color thresholdColorOS = Color.LawnGreen;
		
		
		private DataSeries signal;
		private BoolSeries OB;
		private BoolSeries OS;
		#region Alerts
		private bool generateEmailSignals	= false;
		private bool generateAlerts			= false;
		private string senderEmail			= "";
		private string recipientEmail		= "";
		private string audioFileLong		= "Alert1.wav";
		private string audioFileShort		= "Alert1.wav";
		
		[Description("Generate email signals?")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Generate email signals?")]
		public bool GenerateEmailSignals
        {
            get { return generateEmailSignals; }
			set { generateEmailSignals = value; }
        }
		[Description("Generate alerts?")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Generate alerts?")]
		public bool GenerateAlerts
        {
            get { return generateAlerts; }
			set { generateAlerts = value; }
        }
		[Description("Recipient email?")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Recipient email?")]
		public string RecipientEmail
        {
            get { return recipientEmail; }
			set { recipientEmail = value; }
        }
		[Description("Sender email?")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Sender email?")]
		public string SenderEmail
        {
            get { return senderEmail; }
			set { senderEmail = value; }
        }
		[Description("Audio file for long signals")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Audio file for long signals.")]
		public string AudioFileLong
        {
            get { return audioFileLong; }
			set { audioFileLong = value; }
        }
		[Description("Audio file for short signals")]
        [Category("Alerts")]
		[NinjaTrader.Gui.Design.DisplayName("Audio file for short signals.")]
		public string AudioFileShort
        {
            get { return audioFileShort; }
			set { audioFileShort = value; }
        }
		
		#region Alert Times
		private bool iTradeMonday				= true;
		private bool iTradeTuesday				= true;
		private bool iTradeWednesday			= true;
		private bool iTradeThursday				= true;
		private bool iTradeFriday				= true;
		private bool iTradeSaturday				= false;
		private bool iTradeSunday				= false;
		private int session1Start 				= -1;
		private int session1End 				= -1;
		private int session2Start 				= -1;
		private int session2End 				= -1;
		private int session3Start 				= -1;
		private int session3End 				= -1;
		private bool isTimeToTrade				= false;
		
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("1. Trade on Monday")]
		public bool ITradeMonday
		{
			get { return iTradeMonday; }
			set { iTradeMonday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("2. Trade on Tuesday")]
		public bool ITradeTuesday
		{
			get { return iTradeTuesday; }
			set { iTradeTuesday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("3. Trade on Wednesday")]
		public bool ITradeWednesday
		{
			get { return iTradeWednesday; }
			set { iTradeWednesday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("4. Trade on Thursday")]
		public bool ITradeThursday
		{
			get { return iTradeThursday; }
			set { iTradeThursday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("5. Trade on Friday")]
		public bool ITradeFriday
		{
			get { return iTradeFriday; }
			set { iTradeFriday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("6. Trade on Saturday")]
		public bool ITradeSaturday
		{
			get { return iTradeSaturday; }
			set { iTradeSaturday = value; }
		}        
		[Description("Select if you want to trade on this day of the week.")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("7. Trade on Sunday")]
		public bool ITradeSunday
		{
			get { return iTradeSunday; }
			set { iTradeSunday = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8a. Session1 Start")]
		public int Session1Start
		{
			get { return session1Start; }
			set { session1Start = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8b. Session1 End (-1 to disable)")]
		public int Session1End
		{
			get { return session1End; }
			set { session1End = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8c. Session2 Start (-1 to disable)")]
		public int Session2Start
		{
			get { return session2Start; }
			set { session2Start = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8d. Session2 End (-1 to disable)")]
		public int Session2End
		{
			get { return session2End; }
			set { session2End = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8e. Session3 Start (-1 to disable)")]
		public int Session3Start
		{
			get { return session3Start; }
			set { session3Start = value; }
		}        
		[Description("Session start and stop times are entered with the following format (HHMMSS).")]
		[GridCategory("Alert Time Management")]
		[Gui.Design.DisplayName ("8f. Session3 End (-1 to disable)")]
		public int Session3End
		{
			get { return session3End; }
			set { session3End = value; }
		}        
		#endregion
		#endregion
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Aqua, 2), "STOCH"));					// plot 0

			Add(new Line(thresholdColorOS, thresholdOS, "Lower"));				// line 0
			Add(new Line(thresholdColorOB, thresholdOB, "Upper"));				// line 1
			Add(new Line(Color.LightGray, 0, "Zero"));							// line 2

			Plots[0].Pen.Width = 3;
			Plots[0].Pen.DashStyle = DashStyle.Solid;
			
			den	= new DataSeries(this);
			nom	= new DataSeries(this);
			d = new DataSeries(this);
			k =  new DataSeries(this);
			OB = new BoolSeries(this);
			OS = new BoolSeries(this);
			signal = new DataSeries(this);
			
			Overlay				= false;
            PriceTypeSupported	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			bool canTrade = TimeToTrade();
			Lines[0].Value = thresholdOS;
			Lines[1].Value = thresholdOB;
			Lines[0].Pen.Color = thresholdColorOS;
			Lines[1].Pen.Color = thresholdColorOB;
			
            if (CurrentBar < periodK)
			{
				den.Set(0);
				nom.Set(0);
				STOCH.Set(0);
				d.Set(0);
				k.Set(0);
				OB.Set(false);
				OS.Set(false);
			}
			else
			{
				nom.Set(Close[0] - MIN(Low, PeriodK)[0]);
				den.Set(MAX(High, PeriodK)[0] - MIN(Low, PeriodK)[0]);										// PeriodK = Kperiod
				
				k.Set(RoundPrice(100 * SUM(nom, Smooth)[0] / (SUM(den, Smooth)[0] == 0 ? 1.0 : SUM(den, Smooth)[0])));	// Smooth  = SlowKperiod
				d.Set(RoundPrice(SMA(k, PeriodD)[0]));																	// PeriodD = SlowDperiod
				
				STOCH.Set(k[0]);
				if (k[0] == d[0]) 
				{
					PlotColors[0][0] = Color.Yellow;
					signal.Set(0);
				}
				else if (k[0] > d[0]) 
				{
					PlotColors[0][0] = upColor;
					signal.Set(1);
				}
				else if (k[0] < d[0]) 
				{
					PlotColors[0][0] = downColor;
					signal.Set(-1);
				}
				
				if (STOCH[0] > thresholdOB)
				{
					OB.Set(true);
				}
				else if (STOCH[0] < thresholdOS) 
				{
					OS.Set(true);
				}
				
				//Alerts
				if (signal[0] == 1 && OS[0] && canTrade)
				{
					if (this.generateAlerts)			
						Alert("Stochastic", Priority.High, "Stochastic Long Signal", this.audioFileLong, 30, Color.Black, Color.Green);
					if (this.generateEmailSignals)
						SendMail(this.senderEmail, 
								this.recipientEmail, 
								"Stochastic Long Signal: " + Instrument.MasterInstrument.Name, 
								"Long " + Instrument.MasterInstrument.Name + " at: " + Time[0].ToString());
				}
				else if (signal[0] == -1 && OB[0] && canTrade)
				{
					if (this.generateAlerts)			
						Alert("Stochastic", Priority.High, "Stochastic Short Signal", this.audioFileShort, 30, Color.Black, Color.Red);
					if (this.generateEmailSignals)
						SendMail(this.senderEmail, 
								this.recipientEmail, 
								"Stochastic Short Signal: " + Instrument.MasterInstrument.Name, 
								"Short " + Instrument.MasterInstrument.Name + " at: " + Time[0].ToString());
				}
				
			}
        }

        #region Properties
		/// <summary>
		/// </summary>
		/// 
		
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries D
		{
		get { return d; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries STOCH
		{
		get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries K
		{
		get { return k; }
		}
		
		
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Signal
		{
		get { return signal; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries STOCHOB
		{
		get { return OB; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries STOCHOS
		{
		get { return OS; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for moving average over K values")]
		[GridCategory("Parameters")]
		public int PeriodD
		{
			get { return periodD; }
			set { periodD = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculating the K values")]
		[GridCategory("Parameters")]
		public int PeriodK
		{
			get { return periodK; }
			set { periodK = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing the slow K values")]
		[GridCategory("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		
		[Description("Threshold for overbought line")]
		[Category("Parameters")]
		public double ThresholdOB
		{
		get { return thresholdOB; }
		set { thresholdOB = value; }
		}
		[Description("Threshold for oversold line")]
		[Category("Parameters")]
		public double ThresholdOS
		{
		get { return thresholdOS; }
		set { thresholdOS = value; }
		}
        [XmlIgnore()]		
		[Description("Select color for Stochastic UP")]
		[Category("Visual")]
		[Gui.Design.DisplayName("01. Select color for Stochastic UP")]
		public Color StochasticUPColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string StochasticUPColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [XmlIgnore()]		
		[Description("Select color for Stochastic DOWN")]
		[Category("Visual")]
		[Gui.Design.DisplayName("02. Select color for Stochastic DOWN")]
		public Color StochasticDOWNColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string StochasticDOWNColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [XmlIgnore()]		
		[Description("Select color for Stochastic Overbought Threshold")]
		[Category("Visual")]
		[Gui.Design.DisplayName("03. Select color for Stochastic Overbought Threshold")]
		public Color StochasticOBColor
		{
			get { return thresholdColorOB; }
			set { thresholdColorOB = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string StochasticOBColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(thresholdColorOB); }
			set { thresholdColorOB = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [XmlIgnore()]		
		[Description("Select color for Stochastic Oversold Threshold")]
		[Category("Visual")]
		[Gui.Design.DisplayName("04. Select color for Stochastic Oversold Threshold")]
		public Color StochasticOSColor
		{
			get { return thresholdColorOS; }
			set { thresholdColorOS = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string StochasticOSColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(thresholdColorOS); }
			set { thresholdColorOS = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        #endregion
		
		
		private bool TimeToTrade()
		{
			int block1 = 0;
			int block2 = 0;
			int block3 = 0;
			
			if (session1Start == -1 || session1End == -1)
				block1 = -1;
			if (session2Start == -1 || session2End == -1)
				block2 = -1;
			if (session3Start == -1 || session3End == -1)
				block3 = -1;
			
			if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Monday && this.iTradeMonday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Tuesday && this.iTradeTuesday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Wednesday && this.iTradeWednesday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Thursday && this.iTradeThursday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Friday && this.iTradeFriday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Saturday && this.iTradeSaturday)
				)
				this.isTimeToTrade = true;
			else if (
				(((ToTime(Time[0]) >= session1Start &&  ToTime(Time[0]) < session1End) && block1 != -1)
				||((ToTime(Time[0]) >= session2Start &&  ToTime(Time[0]) < session2End) && block2 != -1)
				||((ToTime(Time[0]) >= session3Start &&  ToTime(Time[0]) < session3End) && block3 != -1)
				|| (block1 == -1 && block2 == -1 && block3 == -1))
				&& (Time[0].DayOfWeek == DayOfWeek.Sunday && this.iTradeSunday)
				)
				this.isTimeToTrade = true;
			else
				this.isTimeToTrade = false;
			
			return this.isTimeToTrade;
		}
	
        private double RoundPrice(double value)
        {
            return Bars.Instrument.MasterInstrument.Round2TickSize(value);
        }
		
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private gemStochasticscolor[] cachegemStochasticscolor = null;

        private static gemStochasticscolor checkgemStochasticscolor = new gemStochasticscolor();

        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        public gemStochasticscolor gemStochasticscolor(bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            return gemStochasticscolor(Input, iTradeFriday, iTradeMonday, iTradeSaturday, iTradeSunday, iTradeThursday, iTradeTuesday, iTradeWednesday, periodD, periodK, session1End, session1Start, session2End, session2Start, session3End, session3Start, smooth, thresholdOB, thresholdOS);
        }

        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        public gemStochasticscolor gemStochasticscolor(Data.IDataSeries input, bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            if (cachegemStochasticscolor != null)
                for (int idx = 0; idx < cachegemStochasticscolor.Length; idx++)
                    if (cachegemStochasticscolor[idx].ITradeFriday == iTradeFriday && cachegemStochasticscolor[idx].ITradeMonday == iTradeMonday && cachegemStochasticscolor[idx].ITradeSaturday == iTradeSaturday && cachegemStochasticscolor[idx].ITradeSunday == iTradeSunday && cachegemStochasticscolor[idx].ITradeThursday == iTradeThursday && cachegemStochasticscolor[idx].ITradeTuesday == iTradeTuesday && cachegemStochasticscolor[idx].ITradeWednesday == iTradeWednesday && cachegemStochasticscolor[idx].PeriodD == periodD && cachegemStochasticscolor[idx].PeriodK == periodK && cachegemStochasticscolor[idx].Session1End == session1End && cachegemStochasticscolor[idx].Session1Start == session1Start && cachegemStochasticscolor[idx].Session2End == session2End && cachegemStochasticscolor[idx].Session2Start == session2Start && cachegemStochasticscolor[idx].Session3End == session3End && cachegemStochasticscolor[idx].Session3Start == session3Start && cachegemStochasticscolor[idx].Smooth == smooth && Math.Abs(cachegemStochasticscolor[idx].ThresholdOB - thresholdOB) <= double.Epsilon && Math.Abs(cachegemStochasticscolor[idx].ThresholdOS - thresholdOS) <= double.Epsilon && cachegemStochasticscolor[idx].EqualsInput(input))
                        return cachegemStochasticscolor[idx];

            lock (checkgemStochasticscolor)
            {
                checkgemStochasticscolor.ITradeFriday = iTradeFriday;
                iTradeFriday = checkgemStochasticscolor.ITradeFriday;
                checkgemStochasticscolor.ITradeMonday = iTradeMonday;
                iTradeMonday = checkgemStochasticscolor.ITradeMonday;
                checkgemStochasticscolor.ITradeSaturday = iTradeSaturday;
                iTradeSaturday = checkgemStochasticscolor.ITradeSaturday;
                checkgemStochasticscolor.ITradeSunday = iTradeSunday;
                iTradeSunday = checkgemStochasticscolor.ITradeSunday;
                checkgemStochasticscolor.ITradeThursday = iTradeThursday;
                iTradeThursday = checkgemStochasticscolor.ITradeThursday;
                checkgemStochasticscolor.ITradeTuesday = iTradeTuesday;
                iTradeTuesday = checkgemStochasticscolor.ITradeTuesday;
                checkgemStochasticscolor.ITradeWednesday = iTradeWednesday;
                iTradeWednesday = checkgemStochasticscolor.ITradeWednesday;
                checkgemStochasticscolor.PeriodD = periodD;
                periodD = checkgemStochasticscolor.PeriodD;
                checkgemStochasticscolor.PeriodK = periodK;
                periodK = checkgemStochasticscolor.PeriodK;
                checkgemStochasticscolor.Session1End = session1End;
                session1End = checkgemStochasticscolor.Session1End;
                checkgemStochasticscolor.Session1Start = session1Start;
                session1Start = checkgemStochasticscolor.Session1Start;
                checkgemStochasticscolor.Session2End = session2End;
                session2End = checkgemStochasticscolor.Session2End;
                checkgemStochasticscolor.Session2Start = session2Start;
                session2Start = checkgemStochasticscolor.Session2Start;
                checkgemStochasticscolor.Session3End = session3End;
                session3End = checkgemStochasticscolor.Session3End;
                checkgemStochasticscolor.Session3Start = session3Start;
                session3Start = checkgemStochasticscolor.Session3Start;
                checkgemStochasticscolor.Smooth = smooth;
                smooth = checkgemStochasticscolor.Smooth;
                checkgemStochasticscolor.ThresholdOB = thresholdOB;
                thresholdOB = checkgemStochasticscolor.ThresholdOB;
                checkgemStochasticscolor.ThresholdOS = thresholdOS;
                thresholdOS = checkgemStochasticscolor.ThresholdOS;

                if (cachegemStochasticscolor != null)
                    for (int idx = 0; idx < cachegemStochasticscolor.Length; idx++)
                        if (cachegemStochasticscolor[idx].ITradeFriday == iTradeFriday && cachegemStochasticscolor[idx].ITradeMonday == iTradeMonday && cachegemStochasticscolor[idx].ITradeSaturday == iTradeSaturday && cachegemStochasticscolor[idx].ITradeSunday == iTradeSunday && cachegemStochasticscolor[idx].ITradeThursday == iTradeThursday && cachegemStochasticscolor[idx].ITradeTuesday == iTradeTuesday && cachegemStochasticscolor[idx].ITradeWednesday == iTradeWednesday && cachegemStochasticscolor[idx].PeriodD == periodD && cachegemStochasticscolor[idx].PeriodK == periodK && cachegemStochasticscolor[idx].Session1End == session1End && cachegemStochasticscolor[idx].Session1Start == session1Start && cachegemStochasticscolor[idx].Session2End == session2End && cachegemStochasticscolor[idx].Session2Start == session2Start && cachegemStochasticscolor[idx].Session3End == session3End && cachegemStochasticscolor[idx].Session3Start == session3Start && cachegemStochasticscolor[idx].Smooth == smooth && Math.Abs(cachegemStochasticscolor[idx].ThresholdOB - thresholdOB) <= double.Epsilon && Math.Abs(cachegemStochasticscolor[idx].ThresholdOS - thresholdOS) <= double.Epsilon && cachegemStochasticscolor[idx].EqualsInput(input))
                            return cachegemStochasticscolor[idx];

                gemStochasticscolor indicator = new gemStochasticscolor();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ITradeFriday = iTradeFriday;
                indicator.ITradeMonday = iTradeMonday;
                indicator.ITradeSaturday = iTradeSaturday;
                indicator.ITradeSunday = iTradeSunday;
                indicator.ITradeThursday = iTradeThursday;
                indicator.ITradeTuesday = iTradeTuesday;
                indicator.ITradeWednesday = iTradeWednesday;
                indicator.PeriodD = periodD;
                indicator.PeriodK = periodK;
                indicator.Session1End = session1End;
                indicator.Session1Start = session1Start;
                indicator.Session2End = session2End;
                indicator.Session2Start = session2Start;
                indicator.Session3End = session3End;
                indicator.Session3Start = session3Start;
                indicator.Smooth = smooth;
                indicator.ThresholdOB = thresholdOB;
                indicator.ThresholdOS = thresholdOS;
                Indicators.Add(indicator);
                indicator.SetUp();

                gemStochasticscolor[] tmp = new gemStochasticscolor[cachegemStochasticscolor == null ? 1 : cachegemStochasticscolor.Length + 1];
                if (cachegemStochasticscolor != null)
                    cachegemStochasticscolor.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachegemStochasticscolor = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.gemStochasticscolor gemStochasticscolor(bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            return _indicator.gemStochasticscolor(Input, iTradeFriday, iTradeMonday, iTradeSaturday, iTradeSunday, iTradeThursday, iTradeTuesday, iTradeWednesday, periodD, periodK, session1End, session1Start, session2End, session2Start, session3End, session3Start, smooth, thresholdOB, thresholdOS);
        }

        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        public Indicator.gemStochasticscolor gemStochasticscolor(Data.IDataSeries input, bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            return _indicator.gemStochasticscolor(input, iTradeFriday, iTradeMonday, iTradeSaturday, iTradeSunday, iTradeThursday, iTradeTuesday, iTradeWednesday, periodD, periodK, session1End, session1Start, session2End, session2Start, session3End, session3Start, smooth, thresholdOB, thresholdOS);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.gemStochasticscolor gemStochasticscolor(bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            return _indicator.gemStochasticscolor(Input, iTradeFriday, iTradeMonday, iTradeSaturday, iTradeSunday, iTradeThursday, iTradeTuesday, iTradeWednesday, periodD, periodK, session1End, session1Start, session2End, session2Start, session3End, session3Start, smooth, thresholdOB, thresholdOS);
        }

        /// <summary>
        /// Colorized Stochastic indicator with one plot and time configurable alerts
        /// </summary>
        /// <returns></returns>
        public Indicator.gemStochasticscolor gemStochasticscolor(Data.IDataSeries input, bool iTradeFriday, bool iTradeMonday, bool iTradeSaturday, bool iTradeSunday, bool iTradeThursday, bool iTradeTuesday, bool iTradeWednesday, int periodD, int periodK, int session1End, int session1Start, int session2End, int session2Start, int session3End, int session3Start, int smooth, double thresholdOB, double thresholdOS)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.gemStochasticscolor(input, iTradeFriday, iTradeMonday, iTradeSaturday, iTradeSunday, iTradeThursday, iTradeTuesday, iTradeWednesday, periodD, periodK, session1End, session1Start, session2End, session2Start, session3End, session3Start, smooth, thresholdOB, thresholdOS);
        }
    }
}
#endregion
